@@ -160,10 +160,11 @@ def expand_schema(schema):
 
 def parse_schema(
     schema,
+    named_schemas=None,
+    *,
     expand=False,
     _write_hint=True,
     _force=False,
-    _named_schemas=None,
 ):
     """Returns a parsed avro schema
 
@@ -175,10 +176,9 @@ def parse_schema(
     ----------
     schema: dict
         Input schema
+    named_schemas: dict
+        Dictionary of named schemas to their schema definition
     expand: bool
-        NOTE: This option should be considered a keyword only argument and may
-        get enforced as such when Python 2 support is dropped.
-
         If true, named schemas will be fully expanded to their true schemas
         rather than being represented as just the name. This format should be
         considered an output only and not passed in to other reader/writer
@@ -190,9 +190,6 @@ def parse_schema(
     _force: bool
         Internal API argument. If True, the schema will always be parsed even
         if it has been parsed and has the __fastavro_parsed marker
-    _named_schemas: dict
-        Internal API argument. Dictionary of named schemas to their schema
-        definition
 
 
     Example::
@@ -203,24 +200,45 @@ def parse_schema(
         parsed_schema = parse_schema(original_schema)
         with open('weather.avro', 'wb') as out:
             writer(out, parsed_schema, records)
+
+
+    Sometimes you might have two schemas where one schema references another.
+    For the sake of example, let's assume you have a `Parent` schema that
+    references a `Child` schema`. If you were to try to parse the parent schema
+    on its own, you would get an exception because the child schema isn't
+    defined. To accomodate this, we can use the `named_schemas` argument to pass
+    a shared dictionary when parsing both of the schemas. The dictionary will
+    get populated with the necessary schema references to make parsing possible.
+    For example::
+
+        from fastavro import parse_schema
+
+        named_schemas = {}
+        parsed_child = parse_schema(child_schema, named_schemas)
+        parsed_parent = parse_schema(parent_schema, named_schemas)
     """
-    if _named_schemas is None:
-        _named_schemas = {}
+    if named_schemas is None:
+        named_schemas = {}
+
+    if isinstance(schema, dict) and "__fastavro_parsed" in schema:
+        for key, value in schema["__named_schemas"].items():
+            named_schemas[key] = value
 
     if _force or expand:
-        return _parse_schema(schema, "", expand, _write_hint, set(), _named_schemas)
+        return _parse_schema(schema, "", expand, _write_hint, set(), named_schemas)
     elif isinstance(schema, dict) and "__fastavro_parsed" in schema:
-        for key, value in schema["__named_schemas"].items():
-            _named_schemas[key] = value
         return schema
     elif isinstance(schema, list):
         # If we are given a list we should make sure that the immediate sub
         # schemas have the hint in them
         return [
-            parse_schema(s, expand, _write_hint, _force, _named_schemas) for s in schema
+            parse_schema(
+                s, named_schemas, expand=expand, _write_hint=_write_hint, _force=_force
+            )
+            for s in schema
         ]
     else:
-        return _parse_schema(schema, "", expand, _write_hint, set(), _named_schemas)
+        return _parse_schema(schema, "", expand, _write_hint, set(), named_schemas)
 
 
 def _parse_schema(schema, namespace, expand, _write_hint, names, named_schemas):
@@ -404,7 +422,7 @@ def parse_field(field, namespace, expand, names, named_schemas):
 
 
 def load_schema(
-    schema_path, *, _named_schemas=None, _write_hint=True, _injected_schemas=None
+    schema_path, *, named_schemas=None, _write_hint=True, _injected_schemas=None
 ):
     """Returns a schema loaded from the file at `schema_path`.
 
@@ -416,9 +434,8 @@ def load_schema(
     ----------
     schema: str
         Path to schema file to load
-    _named_schemas: dict
-        Internal API argument. Dictionary of named schemas to their schema
-        definition
+    named_schemas: dict
+        Dictionary of named schemas to their schema definition
     _write_hint: bool
         Internal API argument specifying whether or not the __fastavro_parsed
         marker should be added to the schema
@@ -460,8 +477,8 @@ def load_schema(
 
         parsed_schema = load_schema("Parent.avsc")
     """
-    if _named_schemas is None:
-        _named_schemas = {}
+    if named_schemas is None:
+        named_schemas = {}
 
     if _injected_schemas is None:
         _injected_schemas = set()
@@ -470,22 +487,20 @@ def load_schema(
         schema = json.load(fd)
     schema_dir, schema_file = path.split(schema_path)
     return _load_schema(
-        schema, schema_dir, _named_schemas, _write_hint, _injected_schemas
+        schema, schema_dir, named_schemas, _write_hint, _injected_schemas
     )
 
 
 def _load_schema(schema, schema_dir, named_schemas, write_hint, injected_schemas):
     try:
         schema_copy = deepcopy(named_schemas)
-        return parse_schema(
-            schema, _named_schemas=named_schemas, _write_hint=write_hint
-        )
+        return parse_schema(schema, named_schemas=named_schemas, _write_hint=write_hint)
     except UnknownType as e:
         try:
             avsc = path.join(schema_dir, f"{e.name}.avsc")
             sub_schema = load_schema(
                 avsc,
-                _named_schemas=schema_copy,
+                named_schemas=schema_copy,
                 _write_hint=False,
                 _injected_schemas=injected_schemas,
             )
@@ -493,7 +508,9 @@ def _load_schema(schema, schema_dir, named_schemas, write_hint, injected_schemas
             raise e
 
         if sub_schema["name"] not in injected_schemas:
-            _inject_schema(schema, sub_schema)
+            injected_schema = _inject_schema(schema, sub_schema)
+            if isinstance(schema, str) or isinstance(schema, list):
+                schema = injected_schema[0]
             injected_schemas.add(sub_schema["name"])
         return _load_schema(
             schema, schema_dir, schema_copy, write_hint, injected_schemas
@@ -644,12 +661,12 @@ def load_schema_ordered(ordered_schemas, *, _write_hint=True):
         )
     """
     loaded_schemas = []
-    _named_schemas = {}
+    named_schemas = {}
     for idx, schema_path in enumerate(ordered_schemas):
         # _write_hint is always False except maybe the outer most schema
         _last = _write_hint if idx + 1 == len(ordered_schemas) else False
         schema = load_schema(
-            schema_path, _named_schemas=_named_schemas, _write_hint=_last
+            schema_path, named_schemas=named_schemas, _write_hint=_last
         )
         loaded_schemas.append(schema)
 
